from flask import Flask, request, jsonify
import requests
from cryptography.fernet import Fernet

# ======== [1. 암호화 키 및 API 키 관리] ======== #
# [보안] 암호화 키는 절대 공개하지 말 것
ENCRYPTION_KEY = b'XxYyZz1234567890abcdef1234567890abcdef1234='  # Fernet 키 (이 키는 안전하게 보관)
ENCRYPTED_API_KEY = b'gAAAAABlQ...=='

# Fernet 객체 생성 및 복호화
cipher = Fernet(ENCRYPTION_KEY)
OPENAI_API_KEY = cipher.decrypt(ENCRYPTED_API_KEY).decode()

# OpenAI 모델 설정
OPENAI_MODEL = "gpt-3.5-turbo"

# Flask 앱 초기화
app = Flask(__name__)

# ======== [2. OpenAI API 호출 함수] ======== #
def get_plant_recommendations(region, temp, humidity):
    prompt = (
        f"다음은 한국 {region} 지역의 기후 조건입니다:\n"
        f"- 온도: {temp}°C\n"
        f"- 습도: {humidity}%\n\n"
        "이 조건에서 잘 자랄 수 있는 식물은 무엇인가요? "
        "한국에서 주로 키우는 대중적인 식물 중 상세한 목록을 제공해 주세요."
    )
    headers = {
        "Authorization": f"Bearer {OPENAI_API_KEY}",
        "Content-Type": "application/json"
    }
    data = {
        "model": OPENAI_MODEL,
        "messages": [
            {"role": "system", "content": "당신은 식물 추천 전문가입니다."},
            {"role": "user", "content": prompt}
        ],
        "max_tokens": 150,
        "temperature": 0.7
    }
    try:
        response = requests.post(
            "https://api.openai.com/v1/chat/completions",
            headers=headers,
            json=data
        )
        response.raise_for_status()
        completion = response.json()
        recommendations = completion['choices'][0]['message']['content'].strip()
        cleaned_recommendations = " ".join(recommendations.splitlines())
        return cleaned_recommendations
    except requests.exceptions.RequestException as e:
        print(f"OpenAI API 요청 오류: {e}")
        return "OpenAI API 요청 실패"
    except KeyError as e:
        print("OpenAI 응답 처리 오류:", e)
        return "OpenAI 응답 처리 오류"


# ======== [3. 추천 엔드포인트] ======== #
@app.route('/recommend', methods=['POST'])
def recommend():
    try:
        user_data = request.json
        measured_temp = user_data["temp"]
        measured_humidity = user_data["humidity"]
        region = "Seoul"
        plant_recommendations = get_plant_recommendations(region, measured_temp, measured_humidity)
        return jsonify({
            "온도": measured_temp,
            "습도": measured_humidity,
            "지역추천": region,
            "식물추천": plant_recommendations
        }), 200
    except Exception as e:
        return jsonify({"error": str(e)}), 500


# ======== [4. Flask 서버 실행] ======== #
if __name__ == '__main__':
    app.run(host='0.0.0.0', port=8080)

